==> ducene-0.0.1.tar.gz: lib/src/index_directory.dart <==
// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:convert';
import 'dart:io';

abstract class IndexDirectory {
  void outputIndex(String name, Map<String,List<int>> index);
  void outputData(String name, Map<int,String> data);
  Map<String,Map<String,List<int>>> inputIndex();
  Map<String,Map<int,String>> inputData(); //TODO lazy
}

class RAMIndexDirectory extends IndexDirectory {

  var _ramIndex = {};
  var _ramData = {};

  void outputIndex(String name, Map<String,List<int>> index) {
    _ramIndex[name] = index;
  }

  void outputData(String name, Map<int,String> data) {
    _ramData[name] = data;
  }

  Map<String,Map<String,List<int>>> inputIndex() => _ramIndex;
  Map<String,Map<int,String>> inputData() => _ramData;

}

class FSIndexDirectory extends IndexDirectory {

  final Directory _dir;

  FSIndexDirectory(this._dir) {
    if (_dir.existsSync()) _dir.deleteSync(recursive: true);
    if (!_dir.existsSync()) _dir.createSync(recursive: true);
  }

  void outputIndex(String name, Map<String,List<int>> index) {
    File indexFile = new File(_dir.path+'/'+name+'.idx');
    if (!indexFile.existsSync()) indexFile.createSync();
    for (String key in index.keys) {
      String value = index[key].join(',');
      String line = key+'=>'+value;
      indexFile.writeAsStringSync(line+'\n', mode: FileMode.APPEND, encoding: UTF8);
    }
  }

  void outputData(String name, Map<int,String> data) {
    File storeFile = new File(_dir.path+'/'+name+'.data');
    if (!storeFile.existsSync()) storeFile.createSync();
    for (int key in data.keys) {
      String value = data[key];
      String line = key.toString()+'=>'+value;
      storeFile.writeAsStringSync(line+'\n', mode: FileMode.APPEND, encoding: UTF8);
    }
  }

  Map<String,Map<String,List<int>>> inputIndex() {
    var ret = {};
    for (String path in _getFileList('.idx')) {
      String name = _getName(path, '.idx');
      File indexFile = new File(path);
      List<String> lines = indexFile.readAsLinesSync();
      var invertedIndex = <String,List<int>>{};
      for (String line in lines) {
        List<String> kv = line.split('=>');
        String key = kv[0];
        List<int> value = kv[1].split(',').map((s) => int.parse(s)).toList();
        invertedIndex[key] = value;
      }
      ret[name] = invertedIndex;
    }
    return ret;
  }

  Map<String,Map<int,String>> inputData() {
    var ret = {};
    for (String path in _getFileList('.data')) {
      String name = _getName(path, '.data');
      File dataFile = new File(path);
      List<String> lines = dataFile.readAsLinesSync();
      var storedData = <int,String>{};
      for (String line in lines) {
        List<String> kv = line.split('=>');
        storedData[int.parse(kv[0])] = kv[1];
      }
      ret[name] = storedData;
    }
    return ret;
  }

  Iterable _getFileList(String extension) =>
      _dir.listSync(recursive: false, followLinks: false)
          .map((e) => e.path)
          .where((path) => path.endsWith(extension));

  String _getName(String path, String extension) =>
      path.split('/').last.replaceAll(extension, '');

}

==> ducene-0.0.2.tar.gz: lib/src/index_directory.dart <==
// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

abstract class IndexDirectory {
  void outputIndex(String name, Map<String,List<int>> index);
  void outputData(String name, Map<int,String> data);
  Map<String,Map<String,List<int>>> inputIndex();
  Map<String,Map<int,String>> inputData(); //TODO lazy
}

class RAMIndexDirectory implements IndexDirectory {

  var _ramIndex = {};
  var _ramData = {};

  void outputIndex(String name, Map<String,List<int>> index) {
    _ramIndex[name] = index;
  }

  void outputData(String name, Map<int,String> data) {
    _ramData[name] = data;
  }

  Map<String,Map<String,List<int>>> inputIndex() => _ramIndex;
  Map<String,Map<int,String>> inputData() => _ramData;

}

class FSIndexDirectory implements IndexDirectory {

  final Directory _dir;

  FSIndexDirectory(this._dir) {
    if (_dir.existsSync()) _dir.deleteSync(recursive: true);
    if (!_dir.existsSync()) _dir.createSync(recursive: true);
  }

  void outputIndex(String name, Map<String,List<int>> index) {
    File indexFile = new File('${_dir.path}/${name}.idx');
    if (!indexFile.existsSync()) indexFile.createSync();
    for (String key in index.keys) {
      String value = index[key].join(',');
      String line = '$key=>$value';
      indexFile.writeAsStringSync(line+'\n', mode: FileMode.APPEND, encoding: UTF8);
    }
  }

  void outputData(String name, Map<int,String> data) {
    File storeFile = new File('${_dir.path}/${name}.data');
    if (!storeFile.existsSync()) storeFile.createSync();
    for (int key in data.keys) {
      String value = data[key];
      String line = '${key.toString()}=>${value}';
      storeFile.writeAsStringSync(line+'\n', mode: FileMode.APPEND, encoding: UTF8);
    }
  }

  Map<String,Map<String,List<int>>> inputIndex() {
    var ret = {};
    for (String path in _getFileList('.idx')) {
      String name = _getName(path, '.idx');
      File indexFile = new File(path);
      List<String> lines = indexFile.readAsLinesSync();
      var invertedIndex = <String,List<int>>{};
      for (String line in lines) {
        List<String> kv = line.split('=>');
        String key = kv[0];
        List<int> value = kv[1].split(',').map((s) => int.parse(s)).toList();
        invertedIndex[key] = value;
      }
      ret[name] = invertedIndex;
    }
    return ret;
  }

  Map<String,Map<int,String>> inputData() {
    var ret = {};
    for (String path in _getFileList('.data')) {
      String name = _getName(path, '.data');
      File dataFile = new File(path);
      List<String> lines = dataFile.readAsLinesSync();
      var storedData = <int,String>{};
      for (String line in lines) {
        List<String> kv = line.split('=>');
        storedData[int.parse(kv[0])] = kv[1];
      }
      ret[name] = storedData;
    }
    return ret;
  }

  Iterable _getFileList(String extension) =>
      _dir.listSync(recursive: false, followLinks: false)
          .map((e) => e.path)
          .where((path) => path.endsWith(extension));

  String _getName(String path, String extension) =>
      path.split('/').last.replaceAll(extension, '');

}
